;***************************************************************************
; 
; File Name		:"TABLES.asm"
; Title			:Constant data
; Date			:98.07.21
; Version		:0.01
; Support telephone	:765 287 1987  David B. VanHorn
; Support fax		:765 287 1989
; Support Email		:dvanhorn@cedar.net
; Target MCU		:AT90S8515
;
;***************************************************************************

;
;	M O D I F I C A T I O N   H I S T O R Y 
;
;       rev.      date    who   why
;	----	--------  ---	------------------------------------------
;	0.01	98.07.29  dvh	Creation
;	0.02	98.09.27  dvh	Added servo min and range tables for fast servo math
;
;
;*************************************************************************
;The provided example could be implemented as an algorithm, simply adding 
;30h to the input byte, but in the table implementation, any value can be 
;used. This is good for functions where the translation algorithm is unknown
;or arbitrary. Mainly I wanted a clear example of table based translation.
;*************************************************************************
;
;.org $F00	;Arbitrary point in romspace
		;Optionally, comment out the .org directive, and
		;the table will float at the end of the code
		;provided it is the LAST FILE included in MAIN
		;
		;Nothing prevents you from splattering your tables
		;throughout the rom space, but it's neater this way :)
.cseg
;
;*************************************************************************
;
;Just an example table for now.
;
;
BIN2ASC_TAB:
;	 0   1   2   3   4   5   6   7   8 
.db	$30,$31,$32,$33,$34,$35,$36,$37,$38
;	 9   A   B   C   D   E   F
.db	$39,$41,$42,$43,$44,$45,$46
;
;*************************************************************************
;
;Example table-reading code. Converts a binary value in TEMP to ASCII.
;To keep it simple we assume that the calling routine will only pass us
;values 0-F, otherwise we will read past the end of the table
;
;Note that this routine does not care where the table is located.
;
;Read_Table:
;	ldi	R30,low(BIN2ASC_TAB*2)	;Make the Z reg point at the table
;	ldi	R31,high(BIN2ASC_TAB*2)	;preparing for the LPM instruction
;	add	R30,TEMP		;Add the calculated offset
;	brcc	RT_A			;If no carry, then we're done
;	inc	R31			;Handle carry
;
;RT_A:	
;	lpm				;look up character
;	ret

;
;I assume external hardware will take care of current control in half step mode
;
; BIG HAIRY NOTE!  Because of the way that the assembler works, and
;the fact that EPROM is 16 bit words, tables built as follows are 
;actually allocating one word per byte, Have a look at the read routine
;in STEPPER.ASM for these tables as opposed to the one above. Both 
;work, but putting more than one byte per .db statement results in
;more efficient use of the romspace.
;
Full_Step_Table:
.db	$08	;1000 Full current
.db	$04	;0100 Full current
.db	$02	;0010 Full current
.db	$01	;0001 Full current

Half_Step_Table:
.db	$08	;1000 Full current
.db	$0C	;1100 Half current
.db	$04	;0100 Full current
.db	$06	;0110 Half current
.db	$02	;0010 Full current
.db	$03	;0011 Half current
.db	$01	;0001 Full current
.db	$09	;1001 Half current

;
;These 16 bit constants are computed at assembly-time, in equates.inc.
;This just places them in ROM so we can extract them easily.
; 
Servo_Min_Table:

.dw	Servo_1_Min
.dw	Servo_2_Min
.dw	Servo_3_Min
.dw	Servo_4_Min
.dw	Servo_5_Min
.dw	Servo_6_Min
.dw	Servo_7_Min
.dw	Servo_8_Min

Servo_Range_Table:

.dw	Servo_1_Range
.dw	Servo_2_Range
.dw	Servo_3_Range
.dw	Servo_4_Range
.dw	Servo_5_Range
.dw	Servo_6_Range
.dw	Servo_7_Range
.dw	Servo_8_Range

Signon_Message:
;	 00000000011111111112222222222333    3          3       3          3        3  3  3  
;	 12345678901234567890123456789012    3          4       5          6        7  8  9
.db	"AVR8515 Startup program version ","MAJOR_REV",".","Minor_Rev","Minor_RevB",CR,LF,NUL
